set echo on feedback on timing on
spool proftest2

drop table proftest_table;

create table proftest_table
(
	c1	number,
	c2	number,
	c3	number,
	c4	number
);

begin
	for i in 1..100000 loop
		insert into proftest_table values (i, 100000-i, i, 100000-i);
	end loop;
end;
/

commit;

create or replace procedure proftest2(in_mode in varchar2, in_table in varchar2)
is
	c		integer;
	rtn		integer;
	v_rowcnt	number;
	v_total_rowcnt	number := 0;
	v_fetches	number := 0;
	v_sql_stmt	varchar2(200);
	--
	v_errcontext	varchar2(250);
	v_errmsg	varchar2(500);
	--
begin
	--
	dbms_application_info.set_module('proftest2', in_table);
	--
	v_errcontext := 'dbms_profiler.start_profiler';
	dbms_profiler.start_profiler(in_mode || ': proftest2(' || in_table || ')');
	--
	v_errcontext := 'construct SQL statement';
	if in_mode = 'SUBSTR' then
		--
		v_sql_stmt := 'select count(substr(ROWID,16,3)) from ' || in_table ||
			' group by substr(ROWID,7,3), substr(ROWID,10,6), substr(ROWID,1,6)';
		--
	elsif in_mode = 'DBMS_ROWID' then
		--
		v_sql_stmt := 'select count(dbms_rowid.rowid_row_number(ROWID)) from ' || in_table ||
			' group by dbms_rowid.rowid_relative_fno(ROWID), dbms_rowid.rowid_block_number(ROWID), dbms_rowid.rowid_object(ROWID)';
		--
	else
		--
		raise_application_error(-20000, 'IN_MODE must be either "SUBSTR" or "DBMS_ROWID"');
		--
	end if;
	--
	v_errcontext := 'dbms_sql.open_cursor';
	c := dbms_sql.open_cursor;
	--
	v_errcontext := substr('dbms_sql.parse("' || v_sql_stmt || '")', 1, 250);
	dbms_sql.parse(c, v_sql_stmt, dbms_sql.native);
	--
	v_errcontext := 'dbms_sql.define_column';
	dbms_sql.define_column(c, 1, v_rowcnt);
	--
	v_errcontext := substr('dbms_sql.execute("' || v_sql_stmt || '")', 1, 250);
	dbms_application_info.set_action(v_errcontext);
	rtn := dbms_sql.execute(c);
	--
	v_errcontext := substr('dbms_sql.fetch_rows("' || v_sql_stmt || '")', 1, 250);
	dbms_application_info.set_action(v_errcontext);
	while dbms_sql.fetch_rows(c) = 1 loop
		--
		v_errcontext := 'dbms_sql.column_value';
		dbms_sql.column_value(c, 1, v_rowcnt);
		--
		v_fetches := v_fetches + 1;
		v_total_rowcnt := v_total_rowcnt + v_rowcnt;
		--
	end loop;
	--
	v_errcontext := 'dbms_sql.close_cursor';
	dbms_sql.close_cursor(c);
	--
	/*
	 * ...drop the rowcount from the very last database block,
	 * so that a partially filled database block does not skew
	 * the average...
	 */
	if v_fetches > 1 then
		--
		v_total_rowcnt := (v_total_rowcnt - v_rowcnt) / (v_fetches - 1);
		--
	end if;
	--
	v_errcontext := 'displaying final report';
	dbms_output.put_line(in_table || ' avgs ' ||
		ltrim(to_char(v_total_rowcnt, '99,990.0000')) ||
		' rows/db-block for ' ||
		ltrim(to_char(v_fetches, '999,999,999,990')) || ' db-blocks');
	--
	v_errcontext := 'dbms_profiler.stop_profiler';
	dbms_profiler.stop_profiler;
	--
exception
	when others then
		v_errmsg := sqlerrm;
		raise_application_error(-20001, v_errcontext || ': ' || v_errmsg);
end proftest2;
/

show errors

spool off
