set echo on feedback on timing on
spool proftest3

create or replace procedure proftest3(in_mode in varchar2)
is
	--
	cursor get_rowids is
	select	rowid
	from	proftest_table
	order by rowid;
	--
	v_blocks	number := 0;
	v_rows		number := 0;
	v_fno		number;
	v_obj		number;
	v_bno		number;
	v_rno		number;
	v_prev_fno	number := -1;
	v_prev_obj	number := -1;
	v_prev_bno	number := -1;
	v_fno_str	varchar2(6);
	v_obj_str	varchar2(6);
	v_bno_str	varchar2(6);
	v_rno_str	varchar2(6);
	v_prev_fno_str	varchar2(6) := '~';
	v_prev_obj_str	varchar2(6) := '~';
	v_prev_bno_str	varchar2(6) := '~';
	--
	v_errcontext	varchar2(250);
	v_errmsg	varchar2(500);
	--
begin
	--
	dbms_application_info.set_module('proftest3', in_mode);
	--
	v_errcontext := 'dbms_profiler.start_profiler';
	dbms_profiler.start_profiler(in_mode || ': proftest3');
	--
	v_errcontext := 'open/fetch get_rowids';
	for x in get_rowids loop
		--
		v_errcontext := 'extracting ROWID components';
		if in_mode = 'DBMS_ROWID' then
			--
			v_fno := dbms_rowid.rowid_relative_fno(x.rowid);
			v_obj := dbms_rowid.rowid_object(x.rowid);
			v_bno := dbms_rowid.rowid_block_number(x.rowid);
			v_rno := dbms_rowid.rowid_row_number(x.rowid);
			--
			if v_fno <> v_prev_fno or
			   v_obj <> v_prev_obj or
			   v_bno <> v_prev_bno
			then
				v_blocks := v_blocks + 1;
			end if;
			--
			v_prev_fno := v_fno;
			v_prev_obj := v_obj;
			v_prev_bno := v_bno;
			--
		else /* in_mode = 'SUBSTR' */
			--
/*
			v_fno_str := substr(rowidtochar(x.rowid), 7, 3);
			v_obj_str := substr(rowidtochar(x.rowid), 10, 6);
			v_bno_str := substr(rowidtochar(x.rowid), 1, 6);
			v_rno_str := substr(rowidtochar(x.rowid), 16, 3);
*/
			v_fno_str := substr(x.rowid, 7, 3);
			v_obj_str := substr(x.rowid, 10, 6);
			v_bno_str := substr(x.rowid, 1, 6);
			v_rno_str := substr(x.rowid, 16, 3);
			--
			if v_fno_str <> v_prev_fno_str or
			   v_obj_str <> v_prev_obj_str or
			   v_bno_str <> v_prev_bno_str
			then
				v_blocks := v_blocks + 1;
			end if;
			--
			v_prev_fno_str := v_fno_str;
			v_prev_obj_str := v_obj_str;
			v_prev_bno_str := v_bno_str;
			--
		end if;
		--
		v_rows := v_rows + 1;
		--
		v_errcontext := 'fetch/close get_rowids';
		--
	end loop; /* end of "get_rowids" cursor loop */
	--
	v_errcontext := 'displaying final report';
	dbms_output.put_line(v_blocks || ' blocks, ' || v_rows || ' rows for an average of ' ||
		trim(to_char(v_rows/v_blocks, '9,990.000')) || ' rows per block');
	--
	v_errcontext := 'dbms_profiler.stop_profiler';
	dbms_profiler.stop_profiler;
	--
exception
	when others then
		v_errmsg := sqlerrm;
		raise_application_error(-20001, v_errcontext || ': ' || v_errmsg);
end proftest3;
/

show errors

spool off
